var assert, events, flow, suite, vows;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
vows = require('vows');
assert = require('assert');
events = require('events');
flow = require('../index');
exports.suite = suite = vows.describe('Heavy Flow');
suite.addBatch({
  'Wait objects': {
    topic: function() {
      return flow.wait;
    },
    'that are empty': {
      topic: function(topic) {
        return topic;
      },
      'should finish immediately after being created': function(topic) {
        var test;
        test = topic();
        return assert.isTrue(test.isFinished());
      },
      'should not be cancelled immediately after being created': function(topic) {
        var test;
        test = topic();
        return assert.isFalse(test.isCancelled());
      }
    },
    'that have no dependencies': {
      topic: function(topic) {
        return topic;
      },
      'should run immediately': function(topic) {
        var ran, test;
        ran = false;
        test = topic(function(finish, cancel) {
          ran = true;
          return finish();
        });
        return assert.isTrue(test.isFinished() && ran);
      },
      'should not be cancelled after finishing successfully': function(topic) {
        var test;
        test = topic(function(finish, cancel) {
          return finish();
        });
        return assert.isFalse(test.isCancelled());
      },
      'should be cancelled after cancelling': function(topic) {
        var test;
        test = topic(function(finish, cancel) {
          return cancel();
        });
        return assert.isTrue(test.isCancelled());
      },
      'should not be finished after cancelling': function(topic) {
        var test;
        test = topic(function(finish, cancel) {
          return cancel();
        });
        return assert.isFalse(test.isFinished());
      },
      'and have callbacks': {
        topic: function(topic) {
          return topic;
        },
        'should call finished callbacks after finishing': function(topic) {
          var called, test;
          called = false;
          test = topic(function(finish, cancel) {
            return finish();
          }).finished(function() {
            return called = true;
          });
          return assert.isTrue(called);
        },
        'should call cancelled callbacks after cancelling': function(topic) {
          var called, test;
          called = false;
          test = topic(function(finish, cancel) {
            return cancel();
          }).cancelled(function() {
            return called = true;
          });
          return assert.isTrue(called);
        },
        'should not call finished callbacks after cancelling': function(topic) {
          var called, test;
          called = false;
          test = topic(function(finish, cancel) {
            return cancel();
          }).finished(function() {
            return called = true;
          });
          return assert.isFalse(called);
        },
        'should not call cancelled callbacks after finishing': function(topic) {
          var called, test;
          called = false;
          test = topic(function(finish, cancel) {
            return finish();
          }).cancelled(function() {
            return called = true;
          });
          return assert.isFalse(called);
        },
        'should pass data to finished functions': function(topic) {
          var data, test;
          data = null;
          test = topic(function(finish, cancel) {
            return finish({
              dumbledore: true
            });
          }).finished(function(result) {
            return data = result;
          });
          return assert.isTrue(data.dumbledore);
        },
        'should pass error messages to cancelled functions': function(topic) {
          var errMsg, test;
          errMsg = '';
          test = topic(function(finish, cancel) {
            return cancel('err');
          }).cancelled(function(err) {
            return errMsg = err;
          });
          return assert.equal(errMsg, 'err');
        }
      }
    },
    'that have synchronous dependencies': {
      topic: function(topic) {
        return topic;
      },
      'should run in order, if none cancel': function(topic) {
        var first, firstRan, second;
        firstRan = false;
        first = topic(function(finish, cancel) {
          firstRan = true;
          return finish();
        });
        second = topic(first, function(finish, cancel) {
          if (firstRan) {
            return finish();
          } else {
            return cancel();
          }
        });
        return assert.isTrue(second.isFinished());
      },
      'should not run if their only dependency cancels': function(topic) {
        var first, second;
        first = topic(function(finish, cancel) {
          return cancel();
        });
        second = topic(first, function(finish, cancel) {
          return finish();
        });
        return assert.isFalse(second.isFinished());
      },
      'should not run if any of their multiple dependencies cancel': function(topic) {
        var first, second, third;
        first = topic(function(finish, cancel) {
          return cancel();
        });
        second = topic(function(finish, cancel) {
          return finish();
        });
        third = topic(first, second, function(finish, cancel) {
          return finish();
        });
        return assert.isFalse(third.isFinished());
      },
      'should run their finished callbacks before calling their dependents': function(topic) {
        var finishedRan, first, second;
        finishedRan = false;
        first = topic().finished(function() {
          return finishedRan = true;
        });
        second = topic(function(finish, cancel) {
          if (finishedRan) {
            return finish();
          } else {
            return cancel();
          }
        });
        return assert.isTrue(second.isFinished());
      },
      'should run their cancellation callbacks before cancelling their dependents': function(topic) {
        var cancelledRan, first, second;
        cancelledRan = false;
        first = topic(function(finish, cancel) {
          return cancel();
        });
        first.cancelled(function(err) {
          return cancelledRan = true;
        });
        return second = topic().cancelled(function(err) {
          return assert.isTrue(cancelledRan);
        });
      },
      'should pass cancellation errors to their dependants': function(topic) {
        var errMsg, first, second;
        first = topic(function(finish, cancel) {
          return cancel('test');
        });
        errMsg = '';
        second = topic(first, function(finish, cancel) {
          return finish();
        }).cancelled(function(msg) {
          return errMsg = msg;
        });
        return assert.equal(errMsg, 'test');
      },
      'should allow optional ready function arguments': {
        topic: function(topic) {
          return topic;
        },
        'should run if their dependency finishes': function(topic) {
          var first, second;
          first = topic();
          second = topic(first);
          return assert.isTrue(second.isFinished());
        },
        'should cancel if their dependency cancels': function(topic) {
          var first, second;
          first = topic(function(finish, cancel) {
            return cancel();
          });
          second = topic(first);
          return assert.isTrue(second.isCancelled());
        },
        'should get the correct error messages if their dependency cancels': function(topic) {
          var errMsg, first, second;
          first = topic(function(finish, cancel) {
            return cancel('test');
          });
          errMsg = '';
          second = topic(first).cancelled(function(msg) {
            return errMsg = msg;
          });
          return assert.equal(errMsg, 'test');
        }
      }
    },
    'that have asynchronous dependencies': {
      topic: function(topic) {
        return topic;
      },
      'that finish': {
        topic: function(topic) {
          var first, firstCalled, second;
          firstCalled = false;
          first = topic(function(finish, cancel) {
            return setTimeout(finish, 500);
          }).finished(function() {
            return firstCalled = true;
          });
          second = topic(first, __bind(function(finish, cancel) {
            if (firstCalled) {
              return this.callback(null, 'success');
            } else {
              return this.callback('err', null);
            }
          }, this));
        },
        'should run in order': function(err, data) {
          return assert.equal(err, null);
        }
      },
      'that cancel': {
        topic: function(topic) {
          var first, second;
          first = topic(function(finish, cancel) {
            return setTimeout(cancel, 500);
          });
          second = topic(first, __bind(function(finish, cancel) {
            this.callback('err', null);
            return finish();
          }, this)).cancelled(__bind(function() {
            return this.callback(null, 'success');
          }, this));
        },
        'should get cancelled': function(err, data) {
          return assert.equal(err, null);
        }
      },
      'that are asynchronous': {
        topic: function(topic) {
          return topic;
        },
        'and have finished functions': {
          topic: function(topic) {
            var test;
            test = topic(function(finish, cancel) {
              return setTimeout(finish, 500);
            }).finished(__bind(function() {
              return this.callback(null, 'success');
            }, this));
          },
          'should finish': function(err, data) {
            return assert.equal(err, null);
          }
        },
        'and have cancelled callbacks, and themselves cancel': {
          topic: function(topic) {
            var test;
            test = topic(function(finish, cancel) {
              return setTimeout(cancel, 500);
            }).cancelled(__bind(function() {
              return this.callback(null, 'success');
            }, this));
          },
          'should call the callbacks': function(err, data) {
            return assert.equal(err, null);
          }
        },
        'and have cancelled callbacks but never cancel': {
          topic: function(topic) {
            var test;
            test = topic(function(finish, cancel) {
              return setTimeout(finish, 500);
            }).cancelled(__bind(function() {
              return this.callback('err', null);
            }, this)).finished(__bind(function() {
              return this.callback(null, 'success');
            }, this));
          },
          'should not call the callbacks': function(err, data) {
            return assert.equal(err, null);
          }
        },
        'and have finished callbacks but never finish': {
          topic: function(topic) {
            var test;
            test = topic(function(finish, cancel) {
              return setTimeout(cancel, 500);
            }).finished(__bind(function() {
              return this.callback('err', null);
            }, this)).cancelled(__bind(function() {
              return this.callback(null, 'success');
            }, this));
          },
          'should not call the callbacks': function(err, data) {
            return assert.equal(err, null);
          }
        },
        'and have finished callbacks added asynchronously': {
          topic: function(topic) {
            var test;
            test = topic(__bind(function(finish, cancel) {
              var addCallback, f;
              addCallback = __bind(function() {
                return test.finished(__bind(function(data) {
                  return this.callback(null, data);
                }, this));
              }, this);
              f = function() {
                finish({
                  dumbledore: true
                });
                return setTimeout(addCallback, 250);
              };
              return setTimeout(f, 250);
            }, this));
          },
          'should call the callbacks': function(err, data) {
            return assert.notEqual(data, null);
          },
          'should pass the correct data': function(err, data) {
            return assert.isTrue(data.dumbledore);
          }
        },
        'and have cancelled callbacks added asynchronously': {
          topic: function(topic) {
            var test;
            test = topic(__bind(function(finish, cancel) {
              var addCallback, c;
              addCallback = __bind(function() {
                return test.cancelled(__bind(function(err) {
                  return this.callback(err, null);
                }, this));
              }, this);
              c = function() {
                cancel('err');
                return setTimeout(addCallback, 250);
              };
              return setTimeout(c, 250);
            }, this));
          },
          'should call the callbacks': function(err, success) {
            return assert.notEqual(err, null);
          },
          'should pass the correct message': function(err, success) {
            return assert.equal(err, 'err');
          }
        }
      }
    }
  }
});